###Указатели  
Иногда возникает необходимость вместо самой переменной получить адрес этой переменной в памяти, 
это может понадобится при работе с массивами, передаче параметров в функцию и для возврата одного или нескольких значений из функции. Поэтому для работы с адресами памяти существуют указатели.  
**Указатель** (англ. **pointer**) - это переменная, которая содержит адрес другой переменной.  
**Адрес оперативной памяти** (англ. **RAM address**) представляет собой число длиной в 4 байта означающее порядковый номер байта в оперативной памяти.  
_Причечание:  
**Байт** - это минимальный объем памяти, который может проадресовать процессор. С/ в оперативную память не может быть,
считан/ записан меньше, чем 1 байт._  
```
type *name = &variable; // Объявление и инициализация указателя
```
`*` является признаком того, что объявляемая переменная - указатель о необычной переменной.  
`&` - это оператор взятия адреса (англ. **address of operator**), позволяет получить адрес указанной переменной.  

`void main() {
  int a = 10; // Объявление переменной a
  int *p; // Объявление указателя
  *p = &a; // Иницализия указателя p адресом переменной a
}
` 

```
void main() {
  int a = 10;  // Объявление переменной a
  int *p = &a;  // Объявление указателя и инициализация его адресом переменной a
  
  cout << a << endl; // Вывод переменной a
  cout << &a << endl; // Вывод адреса переменной a
  cout << p << endl; // Вывод адреса переменной а
  cout << *p << endl; // Вывод значения по адресу переменной a
}
```   

Для того, чтобы вместо адреса переменной обратиться к самой переменной, то есть, получить значение по адресу, нужно разыменовать этот адрес, именно для этого используется **оператор разыменования** (англ. **dereference operator**) - `*`.  
При объявлении указателя `*` показывает, что это указатель и, что в него нужно записать адрес, а не значение. При использовании 
ранее объявленного указателя, `*` позволяет обратится не к адресу, а к значению по адресу.  
А для того, чтобы работать с адресом, хранящимся в указателе `*` нет необходимости.  

При неправильном сочитании операторов `&` и `*` возникает ошибка на этапе компиляции: _`Cannot convert '*int' to 'int'`_   

###Арифметика указателей  
При объявлении указателя обязательно нужно обозначить на переменную какого типа он указывает. Это нужно для того, чтобы компилятор видел какие действия (операции) можно выполнять над переменной, адрес которой содержит указатель и для арифметики указателей.  
Арифметика указателей особенно часто применяется при работе с массивами для того, чтобы получить доступ к определенному элементу. Арифметика указателей жестко привязана к типам данных, а именно к тому, сколько места занимает определенный тип.  

| Тип данных | Размер |
|:----------:|:------:|
| bool       | 1 байт |
| char       | 1 байт |
| short      | 2 байт |
| int        | 4 байт |
| float      | 4 байт |
| long long  | 8 байт |
| double     | 8 байт |

_Если к типу адреса хранящимуся в указателе на `char` прибавить единицу, то этот адрес изменится на 1 байт. Если к указателю на `int` 
прибавить единицу, то адрес хранящийся в нем изменится на 4 байта. К указателям также можно применять операции инкремента и декремента. Арифметика указателей чаще всего нужна для работы с массивами используя указатель._  

###Указатели и массивы  
Имя массива является указателем на массив поскольку содержит адрес нулевого элемента массива. Это легко проверить:  

```
void main() {
  const int s = 5;
  int a[s] = { 3, 14, 8, 2, 5 };
  cout << a << endl;

  for (int i = 0; i < s; ++i)
    cout << *a + i << '\t'; 
    
    }
```  

Сразу происходит разыменование указателя, а потом к значению по адресу прибавляется значение переменной `i`. И таким образом мы получаем сумму нулевого элемента массива и счетчика `i`.  

```
  for (int i = 0; i < s; ++i)
    cout << *a + i << '\t'; 
```  

Для того, чтобы, используя адрес хранящийся в `a` получать доступ к элементам массива,
нужно прибавлять `i` к адресу, а не к нулевому элементу. То есть, нужно сначала вычислить адрес, а потом разыменовать его:  

```
  for (int i = 0; i < s; ++i)
    cout << *(a + i) << '\t'; 
```

